/* -LICENSE-START-
** Copyright (c) 2020 Blackmagic Design
**  
** Permission is hereby granted, free of charge, to any person or organization 
** obtaining a copy of the software and accompanying documentation (the 
** "Software") to use, reproduce, display, distribute, sub-license, execute, 
** and transmit the Software, and to prepare derivative works of the Software, 
** and to permit third-parties to whom the Software is furnished to do so, in 
** accordance with:
** 
** (1) if the Software is obtained from Blackmagic Design, the End User License 
** Agreement for the Software Development Kit (“EULA”) available at 
** https://www.blackmagicdesign.com/EULA/DeckLinkSDK; or
** 
** (2) if the Software is obtained from any third party, such licensing terms 
** as notified by that third party,
** 
** and all subject to the following:
** 
** (3) the copyright notices in the Software and this entire statement, 
** including the above license grant, this restriction and the following 
** disclaimer, must be included in all copies of the Software, in whole or in 
** part, and all derivative works of the Software, unless such copies or 
** derivative works are solely in the form of machine-executable object code 
** generated by a source language processor.
** 
** (4) THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
** OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
** FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT 
** SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE 
** FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE, 
** ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
** DEALINGS IN THE SOFTWARE.
** 
** A copy of the Software is available free of charge at 
** https://www.blackmagicdesign.com/desktopvideo_sdk under the EULA.
** 
** -LICENSE-END-
*/

#pragma once

#include <atomic>
#include "DeckLinkAPI_h.h"

/*
 * An example class which may be used to output a frame or pair of frames to
 * a 3D capable output.
 *
 * This class implements the IDeckLinkMutableVideoFrame interface which can
 * be used to operate on the left frame.
 *
 * Access to the right frame through the IDeckLinkVideoFrame3DExtensions
 * interface:
 *
 * 	IDeckLinkVideoFrame *rightEyeFrame;
 * 	hr = threeDimensionalFrame->GetFrameForRightEye(&rightEyeFrame);
 *
 * After which IDeckLinkVideoFrame operations are performed directly
 * on the rightEyeFrame object.
 */

class SignalGenerator3DVideoFrame : public IDeckLinkMutableVideoFrame, public IDeckLinkVideoFrame3DExtensions
{
public:
	SignalGenerator3DVideoFrame(CComPtr<IDeckLinkMutableVideoFrame>& left, CComPtr<IDeckLinkMutableVideoFrame>& right);
	virtual ~SignalGenerator3DVideoFrame() = default;

	// IUnknown methods
	HRESULT	STDMETHODCALLTYPE QueryInterface(REFIID iid, LPVOID *ppv) override;
	ULONG	STDMETHODCALLTYPE AddRef(void) override;
	ULONG	STDMETHODCALLTYPE Release(void) override;

	// IDeckLinkVideoFrame methods
	long			STDMETHODCALLTYPE GetWidth(void) override;
	long			STDMETHODCALLTYPE GetHeight(void) override;
	long			STDMETHODCALLTYPE GetRowBytes(void) override;
	BMDPixelFormat	STDMETHODCALLTYPE GetPixelFormat(void) override;
	BMDFrameFlags	STDMETHODCALLTYPE GetFlags(void) override;
	HRESULT			STDMETHODCALLTYPE GetBytes(void **buffer) override;

	HRESULT			STDMETHODCALLTYPE GetTimecode (BMDTimecodeFormat format, IDeckLinkTimecode** timecode) override;
	HRESULT			STDMETHODCALLTYPE GetAncillaryData (IDeckLinkVideoFrameAncillary** ancillary) override;

	// IDeckLinkMutableVideoFrame methods
	HRESULT	STDMETHODCALLTYPE SetFlags(BMDFrameFlags newFlags) override;
	HRESULT	STDMETHODCALLTYPE SetTimecode(BMDTimecodeFormat format, IDeckLinkTimecode* timecode) override;
	HRESULT	STDMETHODCALLTYPE SetTimecodeFromComponents(BMDTimecodeFormat format, unsigned char hours, unsigned char minutes, unsigned char seconds, unsigned char frames, BMDTimecodeFlags flags) override;
	HRESULT	STDMETHODCALLTYPE SetAncillaryData(IDeckLinkVideoFrameAncillary* ancillary) override;
	HRESULT	STDMETHODCALLTYPE SetTimecodeUserBits(BMDTimecodeFormat format, BMDTimecodeUserBits userBits) override;

	// IDeckLinkVideoFrame3DExtensions methods
	BMDVideo3DPackingFormat	STDMETHODCALLTYPE Get3DPackingFormat(void) override;
	HRESULT					STDMETHODCALLTYPE GetFrameForRightEye(IDeckLinkVideoFrame** rightEyeFrame) override;

protected:
	CComPtr<IDeckLinkMutableVideoFrame>		m_frameLeft;
	CComPtr<IDeckLinkMutableVideoFrame>		m_frameRight;
	std::atomic<ULONG>						m_refCount;
};

